import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class BitCountsOptionsDialog extends JDialog implements ActionListener, DocumentListener
{
  private int noStates;
  private boolean changed;

  private JLabel noStatesLabel;
  private JTextField noStatesInput;
  private JButton okButton;
  private JButton cancelButton;

  public BitCountsOptionsDialog(JFrame owner)
  {
    super(owner, "Options", true);
    noStatesLabel = new JLabel("No States");
    noStatesInput = new JTextField("" + noStates, 3);
    noStatesInput.getDocument().addDocumentListener(this);
    okButton = new JButton("Ok");
    okButton.addActionListener(this);
    cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(this);
    JPanel basePanel = new JPanel();
    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    JPanel noStatesPanel = new JPanel();
    noStatesPanel.setLayout(new GridLayout(1, 2, 5, 5));
    JPanel noStatesInputPanel = new JPanel();
    noStatesInputPanel.add(noStatesInput);
    noStatesPanel.add(noStatesLabel);
    noStatesPanel.add(noStatesInputPanel);
    noStatesPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), ""));
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    buttonPanel.add(okButton);
    buttonPanel.add(cancelButton);
    basePanel.setLayout(new BorderLayout(0, 5));
    basePanel.add(noStatesPanel, BorderLayout.NORTH);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setResizable(false);
    basePanel.requestFocus();
    changed = false;
  }

  private void setValues()
  {
    try
    {
      int s = new Integer(noStatesInput.getText().trim());
      noStates = s;
      if (noStates < 2)
        noStates = 2;
      if (noStates > 25)
        noStates = 25;
    }
    catch (Exception e)
    {
    }
  }

  public void setData(BitCountsRule rule)
  {
    noStates = rule.getNoStates();
    noStatesInput.setText("" + noStates);
  }

  public void getData(BitCountsRule rule)
  {
    rule.setNoStates(noStates);
    rule.clear();
  }

  public boolean hasChanged()
  {
    return changed;
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == okButton)
    {
      setVisible(false);
      changed = true;
    }
    else if (source == cancelButton)
    {
      setVisible(false);
      changed = false;
    }
  }

  public void insertUpdate(DocumentEvent event)
  {
    setValues();
  }
  
  public void removeUpdate(DocumentEvent event)
  {
    setValues();
  }
  
  public void changedUpdate(DocumentEvent event) {}
}
