import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

class RuleTable2Dialog extends JDialog implements ActionListener
{
  private static final String[][] StateWeightsLabels =
                                           {
                                             {"S0 ", "S8 "},
                                             {"S1 ", "S9 "},
                                             {"S2 ", "S10"},
                                             {"S3 ", "S11"},
                                             {"S4 ", "S12"},
                                             {"S5 ", "S13"},
                                             {"S6 ", "S14"},
                                             {"S7 ", "S15"}
                                           };

  private static final String[][] NeighborWeightsLabels =
                                           {
                                             {"NWNW", "NNW ", "NN  ", "NNE ", "NENE"},
                                             {"WNW ", "NW  ", "N   ", "NE  ", "ENE "},
                                             {"WW  ", "W   ", "C   ", "E   ", "EE  "},
                                             {"WSW ", "SW  ", "S   ", "SE  ", "ESE "},
                                             {"SWSW", "SSW ", "SS  ", "SSE ", "SESE"}
                                           };

  private static final int stateRows = 8;
  private static final int stateColumns = 2;
  private static final int neighborRows = 5;
  private static final int neighborColumns = 5;
  private static final int minimumWeight = -10;
  private static final int maximumWeight = 10;

  private RuleTable2Rule rule;

	private int noStates;
  private int noNeighbors;
  private int noCounts;
  
  private WeightsGrid stateWeightsGrid;
  private WeightsGrid neighborWeightsGrid;
  private RuleTable ruleTable;
  private RuleTable useTable;
  private JPanel ruleTablePanel;
  private JPanel controlPanel;
  private JButton displayButton;
  private JButton randomMergeButton;
  private JButton clearButton;
  private JButton okButton;
  private JButton cancelButton;
  private boolean changed = false;

  public RuleTable2Dialog(JFrame owner, RuleTable2Rule rule, SquareCell squareCell)
  {
    super(owner, "", true);
    setTitle("Rule Table 2");
    this.rule = rule;
		noStates = rule.getNoStates();
    noNeighbors = rule.getNoNeighbors();
    noCounts = rule.getNoCounts();
    JPanel basePanel = new JPanel();
    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    basePanel.setLayout(new BorderLayout());
    JPanel gridPanel = new JPanel();
    Border border = BorderFactory.createEtchedBorder();
    stateWeightsGrid = new WeightsGrid(stateRows, stateColumns, border, "State Weights", StateWeightsLabels,
                                       minimumWeight, maximumWeight);
    gridPanel.add(stateWeightsGrid);
    neighborWeightsGrid = new WeightsGrid(neighborRows, neighborColumns, border, "Neighbor Weights", NeighborWeightsLabels,
                                          minimumWeight, maximumWeight);
    gridPanel.add(neighborWeightsGrid);
    basePanel.add(gridPanel, BorderLayout.NORTH);
    ruleTablePanel = new JPanel();
    ruleTable = new RuleTable(noNeighbors, noCounts, squareCell, true);
    useTable = new RuleTable(noNeighbors, noCounts, squareCell, false);
    ruleTablePanel.add(ruleTable);
    controlPanel = new JPanel();
    controlPanel.setLayout(new GridLayout(3, 1, 5, 5));
    displayButton = new JButton("Use");
    displayButton.addActionListener(this);
    controlPanel.add(displayButton);
    randomMergeButton = new JButton("Random");
    randomMergeButton.addActionListener(this);
    controlPanel.add(randomMergeButton);
    clearButton = new JButton("Clear");
    clearButton.addActionListener(this);
    controlPanel.add(clearButton);
    ruleTablePanel.add(controlPanel);
    ruleTablePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Rule Table"));
    JPanel centerPanel = new JPanel();
    centerPanel.add(ruleTablePanel);
    basePanel.add(centerPanel, BorderLayout.CENTER);
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    okButton = new JButton("Ok");
    okButton.addActionListener(this);
    buttonPanel.add(okButton);
    cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(this);
    buttonPanel.add(cancelButton);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setResizable(false);
    basePanel.requestFocus();
  }

  public void getData(RuleTable2Rule rule)
  {
		for (int i = 0; i < noStates; i++)
			rule.setStateWeight(i, stateWeightsGrid.getValue(i % stateRows, i / stateRows));
    for (int r = 0; r < neighborRows; r++)
      for (int c = 0; c < neighborColumns; c++)
        rule.setNeighborWeight(r, c, neighborWeightsGrid.getValue(r, c));
    for (int r = 0; r < noNeighbors; r++)
      for (int c = 0; c < noCounts; c++)
      {
        rule.setRuleTableValue(r, c, ruleTable.getState(r, c));
        rule.setUseTableValue(r, c, useTable.getState(r, c));
      }
  }

  public void setData(RuleTable2Rule rule)
  {
    for (int r = 0; r < stateRows; r++)
      for (int c = 0; c < stateColumns; c++)
        stateWeightsGrid.setValue(r, c, rule.getStateWeight(c * stateRows + r));
    for (int r = 0; r < neighborRows; r++)
      for (int c = 0; c < neighborColumns; c++)
        neighborWeightsGrid.setValue(r, c, rule.getNeighborWeight(r, c));
    for (int r = 0; r < noNeighbors; r++)
      for (int c = 0; c < noCounts; c++)
      {
        ruleTable.setState(r, c, rule.getRuleTableValue(r, c));
        useTable.setState(r, c, rule.getUseTableValue(r, c));
      }
  }

  public boolean hasChanged()
  {
    return changed;
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == displayButton)
    {
      if (displayButton.getText().equals("Rule"))
      {
        ruleTablePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Rule Table"));
        displayButton.setText("Use");
        randomMergeButton.setText("Random");
        ruleTablePanel.remove(useTable);
        ruleTablePanel.remove(controlPanel);
        ruleTablePanel.add(ruleTable);
        ruleTablePanel.add(controlPanel);
      }
      else
      {
        ruleTablePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Use Table"));
        displayButton.setText("Rule");
        randomMergeButton.setText("Merge");
        ruleTablePanel.remove(ruleTable);
        ruleTablePanel.remove(controlPanel);
        ruleTablePanel.add(useTable);
        ruleTablePanel.add(controlPanel);
      }
      repaint();
    }
    if (source == randomMergeButton)
    {
      if (displayButton.getText().equals("Use"))
        rule.random(ruleTable.getGrid());
      else
        for (int r = 0; r < noNeighbors; r++)
          for (int c = 0; c < noCounts; c++)
            if (useTable.getGrid()[r][c] == 0)
              ruleTable.getGrid()[r][c] = 0;
      repaint();
    }
    else if (source == clearButton)
    {
      if (displayButton.getText().equals("Use"))
        ruleTable.clear();
      else
        useTable.clear();
      repaint();
    }
    else if (source == okButton)
    {
      setVisible(false);
      changed = true;
    }
    else if (source == cancelButton)
    {
      setVisible(false);
      changed = false;
    }
  }
}
