import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

class BitCountsDialog extends JDialog implements ActionListener
{
  private static final String[][] NeighborLabels =
                                        {
                                          {"NW", "N ", "NE"},
                                          {"W ", "C ", "E "},
                                          {"SW", "S ", "SE"}
                                        };
  private static final int noNeighborSelectors = 4;
  private static final int gridSize = 3;

  private BitCountsRule rule;

  private WeightsGrid neighborSelector[];
  private RuleTable ruleTable;
  private JButton randomButton;
  private JButton clearButton;
  private JButton okButton;
  private JButton cancelButton;
  private boolean changed = false;

  public BitCountsDialog(JFrame owner, BitCountsRule rule, SquareCell squareCell)
  {
    super(owner, "Bit Counts", true);
    this.rule = rule;
    JPanel basePanel = new JPanel();
    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    basePanel.setLayout(new BorderLayout());
    JPanel gridPanel = new JPanel();
    Border border = BorderFactory.createEmptyBorder();
    gridPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Neighbor Selector"));
    gridPanel.setLayout(new GridLayout(2, 2));
    neighborSelector = new WeightsGrid[noNeighborSelectors];
    for (int i = 0; i < noNeighborSelectors; i++)
    {
      neighborSelector[i] = new WeightsGrid(gridSize, gridSize, border, "Plane " + i, NeighborLabels, 0, 1);
      gridPanel.add(neighborSelector[i]);
    }
    JPanel gridBasePanel = new JPanel();
    gridBasePanel.add(gridPanel);
    basePanel.add(gridBasePanel, BorderLayout.NORTH);
    JPanel ruleTablePanel = new JPanel();
    ruleTablePanel.setLayout(new BorderLayout());
    ruleTable = new RuleTable(1, rule.getRuleTableSize(), squareCell);
    ruleTablePanel.add(ruleTable, BorderLayout.NORTH);
    JPanel controlPanel = new JPanel();
    controlPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    randomButton = new JButton("Random");
    randomButton.addActionListener(this);
    controlPanel.add(randomButton);
    clearButton = new JButton("Clear");
    clearButton.addActionListener(this);
    controlPanel.add(clearButton);
    ruleTablePanel.add(controlPanel, BorderLayout.SOUTH);
    ruleTablePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Rule"));
    JPanel centerPanel = new JPanel();
    centerPanel.add(ruleTablePanel);
    basePanel.add(centerPanel, BorderLayout.CENTER);
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
    okButton = new JButton("Ok");
    okButton.addActionListener(this);
    buttonPanel.add(okButton);
    cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(this);
    buttonPanel.add(cancelButton);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setResizable(false);
    basePanel.requestFocus();
  }

  public void getData(BitCountsRule rule)
  {
    for (int i = 0; i < noNeighborSelectors; i++)
      for (int r = 0; r < gridSize; r++)
        for (int c = 0; c < gridSize; c++)
          rule.setNeighborSelector(r * gridSize + c, i, neighborSelector[i].getValue(r, c));
    for (int i = 0; i < rule.getRuleTableSize(); i++)
        rule.setRuleTableValue(i, ruleTable.getState(0, i));
  }

  public void setData(BitCountsRule rule)
  {
    for (int i = 0; i < noNeighborSelectors; i++)
      for (int r = 0; r < gridSize; r++)
        for (int c = 0; c < gridSize; c++)
          neighborSelector[i].setValue(r, c, rule.getNeighborSelector(r * gridSize + c, i));
    for (int i = 0; i < rule.getRuleTableSize(); i++)
      ruleTable.setState(0, i, rule.getRuleTableValue(i));
  }

  public void random()
  {
    int maxBitCount = 0;
    for (int i = 0; i < noNeighborSelectors; i++)
      for (int r = 0; r < gridSize; r++)
        for (int c = 0; c < gridSize; c++)
          if (neighborSelector[i].getValue(r, c) == 1)
            maxBitCount++;
    rule.random(ruleTable.getGrid(), maxBitCount);
  }

  public boolean hasChanged()
  {
    return changed;
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == randomButton)
    {
      random();
      repaint();
    }
    else if (source == clearButton)
    {
      ruleTable.clear();
      repaint();
    }
    else if (source == okButton)
    {
      setVisible(false);
      changed = true;
    }
    else if (source == cancelButton)
    {
      setVisible(false);
      changed = false;
    }
  }
}

