import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.util.*;

class Counts2Rule extends Rule
{
  private static final int neighborRows = 5;
  private static final int neighborColumns = 5;

  private int noStates;
  private int ruleTableSize;
  private int percentChange = 30;
  private boolean clearNewRule = true;
  private int neighborWeights[][];
  private int ruleTable[][];

  public Counts2Rule(JFrame mainFrame, SquareCell squareCell)
  {
    super(mainFrame, squareCell);
    setDefault();
  }

  private void makeArrays()
  {
    neighborWeights = new int[neighborRows][neighborColumns];
    ruleTable = new int[ruleTableSize][ruleTableSize];
  }

  private void setWeights()
  {
    for (int r = 0; r < neighborRows; r++)
      for (int c = 0; c < neighborColumns; c++)
        if ((r > 0) && (r < 4) && (c > 0) && (c < 4))
          neighborWeights[r][c] = 1;
        else
          neighborWeights[r][c] = 0;
    neighborWeights[2][2] = 0;
  }

  public void setNewRule()
  {
    random(ruleTable);
  }

  public void random(int ruleTable[][])
  {
    if (clearNewRule)
    {
      for (int r = 0; r < ruleTableSize; r++)
        for (int c = 0; c < ruleTableSize; c++)
          ruleTable[r][c] = 0;
    }
    Random generator = new Random();
    for (int i = 1; i < ruleTableSize * ruleTableSize * percentChange / 100; i++)
    {
      int r = generator.nextInt(ruleTableSize);
      int c = generator.nextInt(ruleTableSize);
      ruleTable[r][c] = generator.nextInt(noStates);
    }
  }

  public void setOptions()
  {
    int currentRuleTableSize = ruleTableSize;
    Counts2OptionsDialog counts2OptionsDialog = new Counts2OptionsDialog(mainFrame);
    counts2OptionsDialog.setData(this);
    centerDialog(counts2OptionsDialog);
    counts2OptionsDialog.setVisible(true);
    if (counts2OptionsDialog.hasChanged())
    {
      counts2OptionsDialog.getData(this);
      if (ruleTableSize != currentRuleTableSize)
        ruleTable = new int[ruleTableSize][ruleTableSize];
    }
  }

  public void setDefault()
  {
    noStates = 8;
    ruleTableSize = 32;
    makeArrays();
    setWeights();
  }

  public void changeRule()
  {
    Counts2Dialog counts2Dialog = new Counts2Dialog(mainFrame, this, squareCell);
    counts2Dialog.setData(this);
    centerDialog(counts2Dialog);
    counts2Dialog.setVisible(true);
    if (counts2Dialog.hasChanged())
      counts2Dialog.getData(this);
  }

  private void centerDialog(JDialog dialog)
  {
    Point p = mainFrame.getLocation();
    int x = (int)p.getX() + (mainFrame.getWidth() - dialog.getWidth()) / 2;
    int y = (int)p.getY() + (mainFrame.getHeight() - dialog.getHeight()) / 2;
    dialog.setLocation(x, y);
  }

  public void writeRule(PrintWriter pw)
  {
    String s;
    pw.println("#Rule = Counts 2");
    pw.println("#States = " + noStates);
    pw.println("#Table = " + ruleTableSize);
    for (int i = 0; i < 5; i++)
    {
      s = "#NW ";
      for (int j = 0; j < 5; j++)
        s = s + neighborWeights[i][j] + ",";
      pw.println(s.substring(0, s.lastIndexOf(",")));
    }
    for (int i = 0; i < ruleTableSize; i++)
    {
      s = "#RT ";
      for (int j = 0; j < ruleTableSize; j++)
        s = s + ruleTable[i][j] + ",";
      pw.println(s.substring(0, s.lastIndexOf(",")));
    }
  }

  public void readRule(BufferedReader br)
  {
    try
    {
      noStates = new Integer(br.readLine().substring(10).trim());
      ruleTableSize = new Integer(br.readLine().substring(9).trim());
      ruleTable = new int[ruleTableSize][ruleTableSize];
      for (int i = 0; i < 5; i++)
      {
        String values[] = br.readLine().substring(3).split(",");
        for (int j = 0; j < 5; j++)
          neighborWeights[i][j] = new Integer(values[j].trim());
      }
      for (int i = 0; i < ruleTableSize; i++)
      {
        String values[] = br.readLine().substring(3).split(",");
        for (int j = 0; j < ruleTableSize; j++)
          ruleTable[i][j] = new Integer(values[j].trim());
      }
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }

  public int step(int r, int c)
  {
    int neighbors[][] = new int[5][5];
    neighbors[0][0] = squareCell.getNeighbor(r - 2, c - 2);
    neighbors[0][1] = squareCell.getNeighbor(r - 2, c - 1);
    neighbors[0][2] = squareCell.getNeighbor(r - 2, c    );
    neighbors[0][3] = squareCell.getNeighbor(r - 2, c + 1);
    neighbors[0][4] = squareCell.getNeighbor(r - 2, c + 2);

    neighbors[1][0] = squareCell.getNeighbor(r - 1, c - 2);
    neighbors[1][1] = squareCell.getNeighbor(r - 1, c - 1);
    neighbors[1][2] = squareCell.getNeighbor(r - 1, c    );
    neighbors[1][3] = squareCell.getNeighbor(r - 1, c + 1);
    neighbors[1][4] = squareCell.getNeighbor(r - 1, c + 2);

    neighbors[2][0] = squareCell.getNeighbor(r    , c - 2);
    neighbors[2][1] = squareCell.getNeighbor(r    , c - 1);
    neighbors[2][2] = squareCell.getNeighbor(r    , c    );
    neighbors[2][3] = squareCell.getNeighbor(r    , c + 1);
    neighbors[2][4] = squareCell.getNeighbor(r    , c + 2);

    neighbors[3][0] = squareCell.getNeighbor(r + 1, c - 2);
    neighbors[3][1] = squareCell.getNeighbor(r + 1, c - 1);
    neighbors[3][2] = squareCell.getNeighbor(r + 1, c    );
    neighbors[3][3] = squareCell.getNeighbor(r + 1, c + 1);
    neighbors[3][4] = squareCell.getNeighbor(r + 1, c + 2);

    neighbors[4][0] = squareCell.getNeighbor(r + 2, c - 2);
    neighbors[4][1] = squareCell.getNeighbor(r + 2, c - 1);
    neighbors[4][2] = squareCell.getNeighbor(r + 2, c    );
    neighbors[4][3] = squareCell.getNeighbor(r + 2, c + 1);
    neighbors[4][4] = squareCell.getNeighbor(r + 2, c + 2);
    int rowCount = 0;
    int columnCount = 0;
    for (r = 0; r < 5; r++)
      for (c = 0; c < 5; c++)
        if (neighbors[r][c] > 0)
        {
          if (neighborWeights[r][c] == 1)
            rowCount += neighbors[r][c];
          else if (neighborWeights[r][c] == 2)
            columnCount += neighbors[r][c];
        }
    if ((rowCount == 0) || (columnCount == 0))
      return 0;
    return ruleTable[(rowCount - 1) % ruleTableSize][(columnCount - 1) % ruleTableSize];
  }

  public int getNoStates()
  {
    return noStates;
  }

  public int getRuleTableSize()
  {
    return ruleTableSize;
  }

  public int getPercentChange()
  {
    return percentChange;
  }

  public boolean getClearNewRule()
  {
    return clearNewRule;
  }

  public int getNeighborWeight(int row, int column)
  {
    return neighborWeights[row][column];
  }

  public int getRuleTableValue(int row, int column)
  {
    return ruleTable[row][column];
  }

  public void setNoStates(int value)
  {
    noStates = value;
  }

  public void setRuleTableSize(int value)
  {
    ruleTableSize = value;
  }

  public void setPercentChange(int percentChange)
  {
    this.percentChange = percentChange;
  }

  public void setClearNewRule(boolean clearNewRule)
  {
    this.clearNewRule = clearNewRule;
  }

  public void setNeighborWeight(int row, int column, int value)
  {
    neighborWeights[row][column] = value;
  }

  public void setRuleTableValue(int row, int column, int value)
  {
    ruleTable[row][column] = value;
  }
}
