import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.util.*;

class RuleTable2Rule extends Rule
{
  private static final int noStateWeights = 16;
  private static final int neighborRows = 5;
  private static final int neighborColumns = 5;

  private int noStates;
  private int noNeighbors;
  private int noCounts;
  private int percentChange = 30;
  private boolean clearNewRule = true;
  private int stateWeights[];
  private int neighborWeights[][];
  private int ruleTable[][];
  private int useTable[][];

  public RuleTable2Rule(JFrame mainFrame, SquareCell squareCell)
  {
    super(mainFrame, squareCell);
    setDefault();
  }

  private void makeArrays()
  {
    stateWeights = new int[noStateWeights];
    neighborWeights = new int[neighborRows][neighborColumns];
    ruleTable = new int[noNeighbors][noCounts];
    useTable = new int[noNeighbors][noCounts];
  }

  private void setWeights()
  {
    for (int r = 1; r < noStates; r++)
      stateWeights[r] = 1;
    for (int r = 0; r < neighborRows; r++)
      for (int c = 0; c < neighborColumns; c++)
        if ((r > 0) && (r < 4) && (c > 0) && (c < 4))
          neighborWeights[r][c] = 1;
        else
          neighborWeights[r][c] = 0;
    neighborWeights[2][2] = 0;
  }

  public void setDefault()
  {
    noStates = 8;
    noNeighbors = 9;
    noCounts = 20;
    makeArrays();
    setWeights();
    for (int r = 0; r < noNeighbors; r++)
      for (int c = 0; c < noCounts; c++)
          ruleTable[r][c] = 0;
  }

  public void changeRule()
  {
    RuleTable2Dialog ruleTable2Dialog = new RuleTable2Dialog(mainFrame, this, squareCell);
    ruleTable2Dialog.setData(this);
    centerDialog(ruleTable2Dialog);
    ruleTable2Dialog.setVisible(true);
    if (ruleTable2Dialog.hasChanged())
		{
      stateWeights = new int[noStateWeights];
      ruleTable2Dialog.getData(this);
		}
  }

  private void centerDialog(JDialog dialog)
  {
    Point p = mainFrame.getLocation();
    int x = (int)p.getX() + (mainFrame.getWidth() - dialog.getWidth()) / 2;
    int y = (int)p.getY() + (mainFrame.getHeight() - dialog.getHeight()) / 2;
    dialog.setLocation(x, y);
  }

  private void clearUseTable()
  {
    for (int r = 0; r < noNeighbors; r++)
      for (int c = 0; c < noCounts; c++)
        useTable[r][c] = 0;
  }

  public void setNewRule()
  {
    random(ruleTable);
  }

  public void random(int ruleTable[][])
  {
    clearUseTable();
    if (clearNewRule)
    {
      for (int r = 0; r < noNeighbors; r++)
        for (int c = 0; c < noCounts; c++)
          ruleTable[r][c] = 0;
    }
    Random generator = new Random();
    for (int i = 1; i < noNeighbors * noCounts * percentChange / 100; i++)
    {
      int r = generator.nextInt(noNeighbors);
      int c = generator.nextInt(noCounts);
      ruleTable[r][c] = generator.nextInt(noStates);
    }
  }

  public void setOptions()
  {
		int currentNoStates = noStates;
    int currentNoNeighbors = noNeighbors;
    int currentNoCounts = noCounts;
    RuleTable2OptionsDialog ruleTable2OptionsDialog = new RuleTable2OptionsDialog(mainFrame);
    ruleTable2OptionsDialog.setData(this);
    centerDialog(ruleTable2OptionsDialog);
    ruleTable2OptionsDialog.setVisible(true);
    if (ruleTable2OptionsDialog.hasChanged())
    {
      ruleTable2OptionsDialog.getData(this);
			if (noStates != currentNoStates)
				for (int i = noStates; i < noStateWeights; i++)
				  stateWeights[i] = 0;
      if ((noNeighbors != currentNoNeighbors) || (noCounts != currentNoCounts))
      {
        ruleTable = new int[noNeighbors][noCounts];
        useTable = new int[noNeighbors][noCounts];
      }
    }
  }

  public void writeRule(PrintWriter pw)
  {
    pw.println("#Rule = Rule Table 2");
    pw.println("#States = " + noStates);
    pw.println("#Neighbors = " + noNeighbors);
    pw.println("#Counts = " + noCounts);
    String s = "#SW ";
    for (int i = 0; i < noStates; i++)
      s = s + stateWeights[i] + ",";
    pw.println(s.substring(0, s.lastIndexOf(",")));
    for (int i = 0; i < 5; i++)
    {
      s = "#NW ";
      for (int j = 0; j < 5; j++)
        s = s + neighborWeights[i][j] + ",";
      pw.println(s.substring(0, s.lastIndexOf(",")));
    }
    for (int i = 0; i < noNeighbors; i++)
    {
      s = "#RT ";
      for (int j = 0; j < noCounts; j++)
        s = s + ruleTable[i][j] + ",";
      pw.println(s.substring(0, s.lastIndexOf(",")));
    }
  }

  public void readRule(BufferedReader br)
  {
    try
    {
      noStates = new Integer(br.readLine().substring(10).trim());
      noNeighbors = new Integer(br.readLine().substring(13).trim());
      noCounts = new Integer(br.readLine().substring(10).trim());
      ruleTable = new int[noNeighbors][noCounts];
      useTable = new int[noNeighbors][noCounts];
      for (int r = 0; r < noStateWeights; r++)
        stateWeights[r] = 0;
      String values[] = br.readLine().substring(3).split(",");
      for (int i = 0; i < noStates; i++)
        stateWeights[i] = new Integer(values[i].trim());
      for (int i = 0; i < 5; i++)
      {
        values = br.readLine().substring(3).split(",");
        for (int j = 0; j < 5; j++)
          neighborWeights[i][j] = new Integer(values[j].trim());
      }
      for (int i = 0; i < noNeighbors; i++)
      {
        values = br.readLine().substring(3).split(",");
        for (int j = 0; j < noCounts; j++)
          ruleTable[i][j] = new Integer(values[j].trim());
      }
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }

  public int step(int r, int c)
  {
    int count = 0;
    int neighborCount = 0;
    for (int i = -2; i <= 2; i++)
      for (int j = -2; j <= 2; j++)
      {
        int k = stateWeights[squareCell.getNeighbor(r + i, c + j)] * neighborWeights[i + 2][j + 2];
        count += k;
        if (k != 0)
          neighborCount++;
      }
    if ((count >= 0) && (count < noCounts) &&
        (neighborCount >= 0) && (neighborCount < noNeighbors))
    {
      useTable[neighborCount][count] = 1;
      return ruleTable[neighborCount][count];
    }
    else
      return 0;
  }

  public int getNoStates()
  {
    return noStates;
  }

  public int getNoNeighbors()
  {
    return noNeighbors;
  }

  public int getNoCounts()
  {
    return noCounts;
  }

  public int getPercentChange()
  {
    return percentChange;
  }

  public boolean getClearNewRule()
  {
    return clearNewRule;
  }

  public int getStateWeight(int state)
  {
    return stateWeights[state];
  }

  public int getNeighborWeight(int row, int column)
  {
    return neighborWeights[row][column];
  }

  public int getRuleTableValue(int row, int column)
  {
    return ruleTable[row][column];
  }

  public int getUseTableValue(int row, int column)
  {
    return useTable[row][column];
  }

  public void setNoStates(int noStates)
  {
    this.noStates = noStates;
  }

  public void setNoNeighbors(int noNeighbors)
  {
    this.noNeighbors = noNeighbors;
  }

  public void setNoCounts(int noCounts)
  {
    this.noCounts = noCounts;
  }

  public void setPercentChange(int percentChange)
  {
    this.percentChange = percentChange;
  }

  public void setClearNewRule(boolean clearNewRule)
  {
    this.clearNewRule = clearNewRule;
  }

  public void setStateWeight(int state, int value)
  {
    stateWeights[state] = value;
  }

  public void setNeighborWeight(int row, int column, int value)
  {
    neighborWeights[row][column] = value;
  }

  public void setRuleTableValue(int row, int column, int value)
  {
    ruleTable[row][column] = value;
  }

  public void setUseTableValue(int row, int column, int value)
  {
    useTable[row][column] = value;
  }
}
